!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ##################
      MODULE MODD_OCEAN_GRID_n
!     ##################
!
!!****  *MODD_OCEAN_GRID_n - declaration of grid for oceanic model
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	C. Lebeaupin Brossier   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2008
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!------------------------------------------------------------------------------
!
TYPE OCEAN_GRID_t
!-------------------------------------------------------------------------------
!
! Grid definition
!
  REAL,POINTER,     DIMENSION(:)  :: XK1,XK2,XK3,XK4 !oceanic lengths between levels
  REAL,POINTER,     DIMENSION(:)  :: XZHOC           !oceanic levels depth
  REAL,POINTER,     DIMENSION(:)  :: XZ2,XDZ1,XDZ2   !oceanic levels and inter-levels
  REAL,POINTER,     DIMENSION(:)  :: XRAY            !solar penetration coef. 
!
!-------------------------------------------------------------------------------
!

END TYPE OCEAN_GRID_t

TYPE(OCEAN_GRID_t), ALLOCATABLE, TARGET, SAVE :: OCEAN_GRID_MODEL(:)


REAL, POINTER, DIMENSION(:) :: XK1
REAL, POINTER, DIMENSION(:) :: XK2
REAL, POINTER, DIMENSION(:) :: XK3
REAL, POINTER, DIMENSION(:) :: XK4
REAL, POINTER, DIMENSION(:) :: XZHOC
REAL, POINTER, DIMENSION(:) :: XZ2
REAL, POINTER, DIMENSION(:) :: XDZ1
REAL, POINTER, DIMENSION(:) :: XDZ2
REAL, POINTER, DIMENSION(:) :: XRAY

CONTAINS

SUBROUTINE OCEAN_GRID_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
OCEAN_GRID_MODEL(KFROM)%XK1=>XK1
OCEAN_GRID_MODEL(KFROM)%XK2=>XK2
OCEAN_GRID_MODEL(KFROM)%XK3=>XK3
OCEAN_GRID_MODEL(KFROM)%XK4=>XK4
OCEAN_GRID_MODEL(KFROM)%XZHOC=>XZHOC
OCEAN_GRID_MODEL(KFROM)%XZ2=>XZ2
OCEAN_GRID_MODEL(KFROM)%XDZ1=>XDZ1
OCEAN_GRID_MODEL(KFROM)%XDZ2=>XDZ2
OCEAN_GRID_MODEL(KFROM)%XRAY=>XRAY
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_OCEAN_GRID_N:OCEAN_GRID_GOTO_MODEL',0,ZHOOK_HANDLE)
XK1=>OCEAN_GRID_MODEL(KTO)%XK1
XK2=>OCEAN_GRID_MODEL(KTO)%XK2
XK3=>OCEAN_GRID_MODEL(KTO)%XK3
XK4=>OCEAN_GRID_MODEL(KTO)%XK4
XZHOC=>OCEAN_GRID_MODEL(KTO)%XZHOC
XZ2=>OCEAN_GRID_MODEL(KTO)%XZ2
XDZ1=>OCEAN_GRID_MODEL(KTO)%XDZ1
XDZ2=>OCEAN_GRID_MODEL(KTO)%XDZ2
XRAY=>OCEAN_GRID_MODEL(KTO)%XRAY
IF (LHOOK) CALL DR_HOOK('MODD_OCEAN_GRID_N:OCEAN_GRID_GOTO_MODEL',1,ZHOOK_HANDLE)
END SUBROUTINE OCEAN_GRID_GOTO_MODEL

SUBROUTINE OCEAN_GRID_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_GRID_N:OCEAN_GRID_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(OCEAN_GRID_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(OCEAN_GRID_MODEL(J)%XK1)
  NULLIFY(OCEAN_GRID_MODEL(J)%XK2)
  NULLIFY(OCEAN_GRID_MODEL(J)%XK3)
  NULLIFY(OCEAN_GRID_MODEL(J)%XK4)
  NULLIFY(OCEAN_GRID_MODEL(J)%XZHOC)
  NULLIFY(OCEAN_GRID_MODEL(J)%XZ2)
  NULLIFY(OCEAN_GRID_MODEL(J)%XDZ1)
  NULLIFY(OCEAN_GRID_MODEL(J)%XDZ2)
  NULLIFY(OCEAN_GRID_MODEL(J)%XRAY)
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_GRID_N:OCEAN_GRID_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE OCEAN_GRID_ALLOC

SUBROUTINE OCEAN_GRID_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_GRID_N:OCEAN_GRID_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(OCEAN_GRID_MODEL)) DEALLOCATE(OCEAN_GRID_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_GRID_N:OCEAN_GRID_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE OCEAN_GRID_DEALLO

END MODULE MODD_OCEAN_GRID_n
