!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ####################################
      SUBROUTINE WRITE_PGD_SURF_ATM_n(HPROGRAM)
!     ####################################
!
!!****  *WRITE_PGD_SURF_ATM_n* - routine to write pgd surface variables 
!!                               in their respective files or in file
!!
!!    PURPOSE
!!    -------
!!
!!**  METHOD
!!    ------
!!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!	B. Decharme   *Meteo France*	
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    05/2011 according to previous write_surf_atmn.f90
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
USE MODD_SURF_CONF,       ONLY : CPROGNAME
USE MODD_SURF_PAR,        ONLY : NVERSION, NBUGFIX
USE MODD_SURF_ATM_GRID_n, ONLY : CGRID, XGRID_PAR, XLAT, XLON, XMESH_SIZE
USE MODD_SURF_ATM_n,      ONLY : NDIM_FULL, NDIM_SEA, NDIM_WATER, NDIM_TOWN, NDIM_NATURE, &
                                 CSEA, CWATER, CNATURE, CTOWN, XCOVER, TTIME, LECOCLIMAP, &
                                 LWATER_TO_NATURE, LTOWN_TO_ROCK, LGARDEN
USE MODD_SURF_ATM_SSO_n,  ONLY : XZ0EFFJPDIR
USE MODD_CH_SURF_n,       ONLY : LCH_EMIS, CCH_EMIS
USE MODD_IO_SURF_FA,      ONLY : LFANOCOMPACT
!
USE MODI_INIT_IO_SURF_n
USE MODI_WRITE_SURF
USE MODI_WRITE_PGD_SEA_n
USE MODI_WRITE_PGD_INLAND_WATER_n
USE MODI_WRITE_PGD_NATURE_n
USE MODI_WRITE_PGD_TOWN_n
USE MODI_END_IO_SURF_n
!
USE MODI_FLAG_UPDATE
!
USE MODI_WRITESURF_COVER_n
USE MODI_WRITESURF_SSO_n
USE MODI_WRITESURF_DUMMY_n
USE MODI_WRITESURF_SNAP_n
USE MODI_WRITESURF_CH_EMIS_n
USE MODI_WRITE_GRID
!
USE MODI_WRITE_ECOCLIMAP2_DATA
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*       0.1   Declarations of arguments
!              -------------------------
!
 CHARACTER(LEN=6),    INTENT(IN)  :: HPROGRAM  ! program calling surf. schemes
!
!*       0.2   Declarations of local variables
!              -------------------------------
!
 CHARACTER(LEN=3)   :: YWRITE
 CHARACTER(LEN=100) :: YCOMMENT
INTEGER            :: IRESP
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-------------------------------------------------------------------------------
!
IF (LHOOK) CALL DR_HOOK('WRITE_PGD_SURF_ATM_N',0,ZHOOK_HANDLE)
!
!*       0.     Initialize some options:
!               ------------------------
!
CPROGNAME = HPROGRAM
!
 CALL FLAG_UPDATE(.FALSE.,.TRUE.,.FALSE.,.FALSE.)
!
!*       1.     Configuration and cover fields:
!               ------------------------------
!
!
!         Initialisation for IO
!
 CALL INIT_IO_SURF_n(HPROGRAM,'FULL  ','SURF  ','WRITE')
!
YWRITE='PGD'
YCOMMENT='(-)'
 CALL WRITE_SURF(HPROGRAM,'VERSION',NVERSION,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'BUG    ',NBUGFIX ,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'STORAGETYPE',YWRITE,IRESP,YCOMMENT)
!
 CALL WRITE_SURF(HPROGRAM,'SEA   ',CSEA   ,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'WATER ',CWATER ,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'NATURE',CNATURE,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'TOWN  ',CTOWN  ,IRESP,YCOMMENT)
!
 CALL WRITE_SURF(HPROGRAM,'DIM_FULL  ',NDIM_FULL,  IRESP,HCOMMENT=YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'DIM_SEA   ',NDIM_SEA,   IRESP,HCOMMENT=YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'DIM_NATURE',NDIM_NATURE,IRESP,HCOMMENT=YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'DIM_WATER ',NDIM_WATER, IRESP,HCOMMENT=YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'DIM_TOWN  ',NDIM_TOWN,  IRESP,HCOMMENT=YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'ECOCLIMAP ',LECOCLIMAP ,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'WATER_TO_NAT',LWATER_TO_NATURE,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'TOWN_TO_ROCK',LTOWN_TO_ROCK,IRESP,YCOMMENT)
 CALL WRITE_SURF(HPROGRAM,'GARDEN',LGARDEN,IRESP,YCOMMENT)
IF (HPROGRAM.NE.'BINARY' .AND. HPROGRAM.NE.'TEXTE ') THEN
   CALL WRITE_ECOCLIMAP2_DATA(HPROGRAM)
ENDIF
!
 CALL WRITE_GRID(HPROGRAM,CGRID,XGRID_PAR,XLAT,XLON,XMESH_SIZE,IRESP,XZ0EFFJPDIR)
!
 CALL WRITESURF_COVER_n(HPROGRAM)
 CALL WRITESURF_SSO_n(HPROGRAM)
 CALL WRITESURF_DUMMY_n(HPROGRAM)
!
YCOMMENT='CH_EMIS'
 CALL WRITE_SURF(HPROGRAM,'CH_EMIS',LCH_EMIS,IRESP,HCOMMENT=YCOMMENT)
!
IF (LCH_EMIS) THEN
  YCOMMENT='CH_EMIS_OPT'
  CALL WRITE_SURF(HPROGRAM,'CH_EMIS_OPT',CCH_EMIS,IRESP,HCOMMENT=YCOMMENT)
END IF
!
IF (LCH_EMIS) THEN
  IF (CCH_EMIS=='AGGR') THEN
    CALL WRITESURF_CH_EMIS_n(HPROGRAM)
  ELSE IF (CCH_EMIS=='SNAP') THEN
    CALL WRITESURF_SNAP_n(HPROGRAM)
  ENDIF
ENDIF
!
!         End of IO
!
 CALL END_IO_SURF_n(HPROGRAM)
!
!
!*       2.     Sea
!               ---
!
IF (NDIM_SEA>0) CALL WRITE_PGD_SEA_n(HPROGRAM)
!
!
!*       3.     Inland water
!               ------------
!
IF (NDIM_WATER>0) CALL WRITE_PGD_INLAND_WATER_n(HPROGRAM)
!
!
!*       4.     Vegetation scheme
!               -----------------
!
IF (NDIM_NATURE>0) CALL WRITE_PGD_NATURE_n(HPROGRAM)
!
!
!*       5.     Urban scheme
!               ------------
!
IF (NDIM_TOWN>0) CALL WRITE_PGD_TOWN_n(HPROGRAM)
!
!
IF (LHOOK) CALL DR_HOOK('WRITE_PGD_SURF_ATM_N',1,ZHOOK_HANDLE)
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE WRITE_PGD_SURF_ATM_n
